/*
 * Utility.h
 *
 *  Created on: Dec 22, 2014
 *      Author: mohajerani
 */

#ifndef UTILITY_H_
#define UTILITY_H_

#include <math.h>
#include <stdio.h>
#include <stdlib.h>

/* ---------------------------------------------------------------------------------
 * Math constants.
 */
#define PI              3.14159265358979
#define TWO_PI          6.28318530717959
#define PI_HALF         1.5707963267949
#define PI_THIRD        1.0471975511966
#define PI_SIXTH        0.523598775598299
#define INV_PI          0.318309886183791
#define INV_PI_HALF     0.636619772367581
#define INV_TWO_PI      0.159154943091895
#define INV_PI_THIRD    0.954929658551372

#define SQRT_TWO        1.4142135623731
#define SQRT_THREE      1.73205080756888
#define SQRT_THREE_INV  0.577350269189626
#define TWO_SQRT_THREE  3.46410161513775
#define ONE_THIRD       0.333333333333333

#define limitToMin(varMin, var) var = (var < varMin)?  varMin : var;
#define limitToMax(varMax, var) var = (var > varMax)?  varMax : var;

/* ---------------------------------------------------------------------------------
 * Enumerated types.
 */
typedef enum {
  LPF = 1,
  HPF = 2,
  BPF = 3,
  NOTCH = 4
} FilterType;

typedef enum {
  PM = 1,
  IM = 2
} MotorType;

/* ---------------------------------------------------------------------------------
 * Structures.
 */
typedef struct {
  // Natural frequency and damping ratio.
	float wn, zeta;
	// numerator coefficients
	float a0, a1, a2;
	// denumerator coefficients
	float b0Inverse, b0, b1, b2;
	// input/output variables from previous time steps
	float x2, x1, y2, y1;
  // filter type.
  FilterType type;
} Filter;

typedef struct {
  // Gains.
  float Kp, Ki, Kd;
  // i-term memory.
  float iTerm;
  // Sampling time.
  float Ts;
} PID;

typedef struct {
  // Stator resistance.
  float Rs;
  // Pole pair number.
  float polePair;
  // Static inductances.
  float LdStatic, LqStatic;
  // Dynamic inductances.
  float LdDynamic, LqDynamic;
  // Friction.
  float B;
  // Inertia.
  float J;
  // Motor type.
  MotorType type;
} Motor;

/* ---------------------------------------------------------------------------------
 * Digital filter.
 */
void digitalFilterInit(FilterType type, float cutoffFreq, float dampRatio, float Ts, int memReset, Filter *filt);
void digitalFilter(float var, float* varFilt, Filter* filt);

#ifdef CONTROLLER_BUILD
/* ---------------------------------------------------------------------------------
 * Limits the angle in [0, 2*pi].
 */
inline float thetaWrap(float theta) {
	float thetaNew = 0;
	long int thetaInt = (long int) (theta * INV_TWO_PI);
    if(theta < 0.0)
		thetaNew = theta + (thetaInt + 1) * TWO_PI;
	else
		thetaNew = theta - thetaInt * TWO_PI;

	return thetaNew;
}
#else
#if UTILITY_C
float thetaWrap(float theta) {
  float thetaNew = 0;
  long int thetaInt = (long int) (theta  * INV_TWO_PI);

  if(theta < 0.0)
    thetaNew = theta + (thetaInt + 1) * TWO_PI;
  else
    thetaNew = theta - thetaInt * TWO_PI;

  return thetaNew;
}
#else
extern float thetaWrap(float theta);
#endif
#endif

/* ---------------------------------------------------------------------------------
 * Calculates the sector number and sector angle.
 */
void sectorCalc(float theta, float* thetaSector, int* sectorNum);

/* ---------------------------------------------------------------------------------
 * Converts from "alpha and betta" to "norm and angle".
 */
void alphaBetaToNormAngle(float alpha, float beta, float* theta, float* norm);

/* ---------------------------------------------------------------------------------
 * dq to abc transformation.
 */
void dqToABC(float d, float q, float theta, float* a, float* b, float* c);

/* ---------------------------------------------------------------------------------
 * abc to dq transformation.
 */
void abcToDQ(float a, float b, float c, float theta, float* d, float* q);

/* ---------------------------------------------------------------------------------
 * Conducts binary search to find the neighboring elements of x in vecX.
 * Calculates a linearly interpolated value for y from vecY corresponding to the above elements
 */
float linearInterpol(float vecX[], float vecY[], int vecLength, float x);

/* ---------------------------------------------------------------------------------
 *
 */
float tableLookup(float vecX[], float vecY[], int vecLength, float invStepX, float x) ;

/* ---------------------------------------------------------------------------------
 * Rotates a vector with [x,y] coordinates theta_rot degrees in CCW direction.
 */
void rotate(float x, float y, float thetaRot, float* xRot, float* yRot) ;

/* ---------------------------------------------------------------------------------
 * Limits the minimum and maximum values to varMin and varMax.
 * var is both input and output.
 */
void limitToMinMax(float varMin, float varMax, float* var);

/* ---------------------------------------------------------------------------------
 * PI controller.
 */
void controllerInitPI(float pGain, float iGain, float Ts, PID* controller);
void controllerPI(float in, float* out, PID* controller);

#endif /* UTILITY_H_ */
