/*
 * Resolver.c
 *
 *  Created on: Jan 13, 2015
 *      Author: mohajerani
 */

#include "Resolver.h"

/* ---------------------------------------------------------------------------------
 *
 */
void resolverOffsetInit(float voltHF, float omegaHF, float dampRatio, float cutoffLPF, float iGain, float pGain, float dGain, float Ts,
                        Motor mtrParam, float* demodGain, Filter* resolverLPF, Filter* resolverBPF, SpeedObserver* speedObserver) {
  // Demodulation gain initialization.
  float inductorGain = (mtrParam.LqDynamic - mtrParam.LdDynamic) / (mtrParam.LqDynamic * mtrParam.LdDynamic) / 4.0;
  *demodGain = omegaHF / voltHF / inductorGain;

  // Observer initialization.
  speedObserverInit(pGain, iGain, dGain, Ts, mtrParam, speedObserver);

  // Low Pass Filter (LPF) initialization.
  digitalFilterInit(LPF, cutoffLPF, dampRatio, Ts, 1, resolverLPF);

  // Band Pass Filter (BPF) initialization.
  digitalFilterInit(BPF, omegaHF * INV_TWO_PI, dampRatio, Ts, 1, resolverBPF);
}

/* ---------------------------------------------------------------------------------
 *
 */
void resolverOffset(float voltHF, float omegaHF, float Ts, float thetaResolver, float demodGain, float electricTrq, float* thetaOffset, float* thetaEst,
                    float* speedEst, float* torqueEst, float* iABC, float* vCmdABC, Filter* resolverLPF, Filter* resolverBPF, SpeedObserver* speedObs) {
  // High frequency voltage injection.
  float sinHF = 0;
  highVoltageInjection(voltHF, omegaHF, *thetaEst, Ts, &sinHF, vCmdABC);

  // Demodulation.
  float dTheta = 0;
  float id, iq;
  abcToDQ(iABC[0], iABC[1], iABC[2], *thetaEst, &id, &iq);
  resolverOffsetDemodulation(iq, sinHF, demodGain, &dTheta, resolverLPF, resolverBPF);

  // Speed observer.
  speedObserver(dTheta, Ts, electricTrq, speedEst, thetaEst, torqueEst, speedObs);

  *thetaOffset = *thetaEst - thetaResolver;
}
/* ---------------------------------------------------------------------------------
 *
 */
void highVoltageInjection(float voltHF, float omegaHF, float theta, float Ts, float* sinHF, float* vCmdABC) {
    static float time = 0;
    float cosHF = cos(time * omegaHF);
    float vdHF = voltHF * cosHF;
    float vaHF, vbHF, vcHF;

    dqToABC(vdHF, 0, theta, &vaHF, &vbHF, &vcHF);

    vCmdABC[0] = vaHF;
    vCmdABC[1] = vbHF;
    vCmdABC[2] = vcHF;

    *sinHF = sin(time * omegaHF);
    time += Ts;
}

/* ---------------------------------------------------------------------------------
 *
 */
void resolverOffsetDemodulation(float iq, float sinHF, float demodGain, float* dTheta, Filter* resolverLPF, Filter* resolverBPF) {
  float iqHF = 0;
  digitalFilter(iq, &iqHF, resolverBPF);

  float err = iqHF * sinHF;
  float errFilt = 0;
  digitalFilter(err, &errFilt, resolverLPF);

  *dTheta = demodGain * errFilt;
}

/* ---------------------------------------------------------------------------------
 *
 */
void speedObserverInit(float pGain, float iGain, float dGain, float Ts, Motor mtrParam, SpeedObserver* obs) {
  obs->Ts = Ts;
  obs->Kp = pGain;
  obs->Ki = iGain;
  obs->Kd = dGain;
  obs->KpSpeed = mtrParam.B;
  obs->KiSpeed = 1/mtrParam.J;

  obs->iTermAngle = 0.0;
  obs->iTermSpeed = 0.0;
  obs->iTermTorque = 0.0;
}

/* ---------------------------------------------------------------------------------
 *
 */
void speedObserver(float dTheta, float Ts, float eTrq, float* omegaEst, float* thetaEst, float* torqueEst, SpeedObserver* obs) {
  // Torque estimation.
  obs->iTermTorque += obs->Ts * dTheta;
  *torqueEst = obs->Ki * obs->iTermTorque;

  // Speed estimation.
  float speedOld = obs->KiSpeed * obs->iTermSpeed;
  float dOmegaEst = eTrq + *torqueEst + obs->Kp * dTheta - obs->KpSpeed * speedOld;
  obs->iTermSpeed += obs->Ts * dOmegaEst;
  *omegaEst = obs->KiSpeed * obs->iTermSpeed;

  // Angle estimation.
  float dThetaEst = obs->Kd * dTheta + *omegaEst;
  obs->iTermAngle += obs->Ts * dThetaEst;
  *thetaEst = obs->iTermAngle;
}


