/*
 * Utility
 *
 *  Created on: Dec 22, 2014
 *      Author: mohajerani
 */

#define UTILITY_C 1
#include "Mathlib.h"

/* ---------------------------------------------------------------------------------
 *
 */
void sectorCalc(float theta, float* thetaSector, int* sectorNum) {
	theta = thetaWrap(theta);

	*sectorNum = theta * INV_PI_THIRD + 1;
	*thetaSector = theta - (*sectorNum - 1) * PI_THIRD;
}

/* ---------------------------------------------------------------------------------
 *
 */
void alphaBetaToNormAngle(float alpha, float beta, float* theta, float* norm) {
	// norm ---------
	*norm = __sqrt(alpha * alpha + beta * beta);

	// theta --------
    float thetaTemp = acosTbl(alpha/ *norm);
    
    if (beta < 0.0) {
		thetaTemp = TWO_PI - thetaTemp;
	} 
    
    *theta = thetaWrap(thetaTemp);
}

/* ---------------------------------------------------------------------------------
 *
 */
void dqToABC(float d, float q, float theta, float* a, float* b, float* c) {

  float  sinTblTheta = sinTbl(theta);
  float  cosTblTheta = cosTbl(theta);

  float dsintbl = d * sinTblTheta;
  float qsintbl = q * sinTblTheta;
  float dcosTbl = d * cosTblTheta;
  float qcosTbl = q * cosTblTheta;

  float alpha = dcosTbl - qsintbl;
  float beta  = dsintbl + qcosTbl;

  *a = alpha;
  *b = -0.5 * (alpha - beta * SQRT_THREE);
  *c = -0.5 * (alpha + beta * SQRT_THREE);
}
/* ---------------------------------------------------------------------------------
 *
 */
void abcToDQ(float a, float b, float c, float theta, float* d, float* q) {

  float alpha = ONE_THIRD * (2.0 * a - b - c);
  float beta  = SQRT_THREE_INV * (b - c);

  float sinTblTheta = sinTbl(theta);
  float cosTblTheta = cosTbl(theta);

  float asintbl = alpha * sinTblTheta;
  float acosTbl = alpha * cosTblTheta;
  float bsintbl = beta * sinTblTheta;
  float bcosTbl = beta * cosTblTheta;

  *d = acosTbl + bsintbl;
  *q = -asintbl + bcosTbl;
}

/* ---------------------------------------------------------------------------------
 *
 */
float tableLookup(float vecX[], float vecY[], int vecLength, float invStepX, float x) {
  // Special condition:
  if (vecLength == 1) {
    return vecY[0];
  }

  int i = (x - vecX[0]) * invStepX;
  if (i < 0) {
    return vecY[0];
  } else if  (i >= vecLength - 1) {
    return vecY[vecLength - 1];
  } else {
    return vecY[i] + (vecY[i + 1] - vecY[i]) * (x - vecX[i]) / (vecX[i + 1] - vecX[i]);
  }
}
/* ---------------------------------------------------------------------------------
 *
 */
float linearInterpol(float vecX[], float vecY[], int vecLength, float x) {
	// Special condition:
	if (vecLength == 1) {
		return vecY[0];
	}

	// Normal case:
	int head = 0;
	int tail = vecLength - 1;

	while(1) {
		if (x <= vecX[head]) {
			return vecY[head];
		}
		if (x >= vecX[tail]) {
			return vecY[tail];
		}
		if (tail == head + 1) {
			return vecY[head] + (vecY[tail] - vecY[head]) * (x - vecX[head]) / (vecX[tail] - vecX[head]);
		}

		int mid = (head + tail) * 0.5;
		if (x <= vecX[mid]) {
			tail = mid;
		} else {
			head = mid;
		}
	}

	//return -1.0;
}

/* ---------------------------------------------------------------------------------
 *
 */
void rotate(float x, float y, float thetaRot, float* xRot, float* yRot) {

  float sinTblTheta = sinTbl(thetaRot);
  float cosTblTheta = cosTbl(thetaRot);

	float xsintbl = x * sinTblTheta;
	float ysintbl = y * sinTblTheta;
	float xcosTbl = x * cosTblTheta;
	float ycosTbl = y * cosTblTheta;

	*xRot = xcosTbl - ysintbl;
	*yRot = xsintbl + ycosTbl;
}

/* ---------------------------------------------------------------------------------
 *
 */
void limitToMinMax(float varMin, float varMax, float* var) {
	limitToMax(varMax, *var);
	limitToMin(varMin, *var);
}

/* ---------------------------------------------------------------------------------
 *
 */
void digitalFilterInit(FilterType type, float cutoffFreq, float dampRatio, float Ts, int memReset, Filter *filt) {
  float c = 1; // c = 1 or 2/Ts
	float wnDigital = c * tan(PI * Ts * cutoffFreq);

	float a0 = 0;
	float a1 = 0;
	float a2 = 0;

	float b0 = 1.0;
	float b1 = 2.0 * dampRatio / wnDigital;
	float b2 = 1.0 / (wnDigital * wnDigital);

	// analogue filter coefficients.
	switch (type) {
		case NOTCH:
			a0 = 1.0;
			a2 = -1.0 / (wnDigital * wnDigital);
			break;
		case HPF:
			a2 = 1.0 / (wnDigital * wnDigital);
			break;
		case BPF:
			a1 = 2 * dampRatio / wnDigital;
			break;
		case LPF:
		default:
			a0 = 1.0;
			break;
		}

	filt->type = type;
  filt->zeta = dampRatio;
	filt->wn = TWO_PI * cutoffFreq;

	// Digital filter coefficients.
	// Bilinear transformation: s = c * (z-1)/(z+1).
	float cSquare = c * c;

  filt->b0Inverse = 1.0 / (b0 + b1 * c + b2 * cSquare);
	filt->b2 = (b0 - b1 * c + b2 * cSquare) * filt->b0Inverse;
	filt->b1 = (2 * (b0 - b2 * cSquare)) * filt->b0Inverse;
	filt->b0 = 1.0;

  filt->a2 = (a0 - a1 * c + a2 * cSquare) * filt->b0Inverse;
  filt->a1 = (2 * (a0 - a2 * cSquare)) * filt->b0Inverse;
  filt->a0 = (a0 + a1 * c + a2 * cSquare) * filt->b0Inverse;

  // Reset memory.
  if (memReset > 0) {
    filt->y1 = 0.0;
    filt->y2 = 0.0;
    filt->x1 = 0.0;
    filt->x2 = 0.0;
  }
}

/* ---------------------------------------------------------------------------------
 *
 */
void digitalFilter(float var, float* varFilt, Filter* filt) {

	*varFilt = filt->a2 * filt->x2 + filt->a1 * filt->x1 + filt->a0 * var
	          -filt->b2 * filt->y2 - filt->b1 * filt->y1;

	filt->x2 = filt->x1;
	filt->x1 = var;
	filt->y2 = filt->y1;
	filt->y1 = *varFilt;
}

/* ---------------------------------------------------------------------------------
 *
 */
void controllerPI(float in, float* out, PID* controller) {
  controller->iTerm += controller->Ts * in;
  *out = controller->Kp * in + controller->Ki * controller->iTerm;
}

/* ---------------------------------------------------------------------------------
 *
 */
void controllerInitPI(float pGain, float iGain, float Ts, PID* controller) {
  controller->Ts = Ts;
  controller->Kp = pGain;
  controller->Ki = iGain;
  controller->Kd = 0.0;
  controller->iTerm = 0.0;
}

#undef UTILITY_C
